@@ -8,6 +8,7 @@
 
 struct proxyvars {
    bool ignorecr;
+   char *version;
    bool doresolve;
    char *authstring;
    char *authfile;
@@ -17,16 +18,15 @@ struct proxyvars {
 
 extern const struct optdesc opt_proxyport;
 extern const struct optdesc opt_ignorecr;
+extern const struct optdesc opt_http_version;
 extern const struct optdesc opt_proxy_resolve;
 extern const struct optdesc opt_proxy_authorization;
 extern const struct optdesc opt_proxy_authorization_file;
 
-extern const struct addrdesc addr_proxy_connect;
+extern const struct addrdesc xioaddr_proxy_connect;
 
-int _xioopen_proxy_prepare(struct proxyvars *proxyvars, struct opt *opts,
-			   const char *targetname, const char *targetport);
-int _xioopen_proxy_connect(struct single *xfd,
-			   struct proxyvars *proxyvars,
-			   int level);
+extern int _xioopen_proxy_init(struct proxyvars *proxyvars, struct opt *opts, const char *targetname, const char *targetport);
+extern int _xioopen_proxy_prepare(struct proxyvars *proxyvars, struct opt *opts, const char *targetname, const char *targetport, const int ai_flags[2]);
+extern int _xioopen_proxy_connect(struct single *xfd, struct proxyvars *proxyvars, int level);
 
 #endif /* !defined(__xio_proxy_h_included) */
