@@ -1,5 +1,5 @@
 /* source: xio-readline.c */
-/* Copyright Gerhard Rieger 2002-2009 */
+/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening the readline address */
@@ -25,29 +25,32 @@ uses stdin!!
 /* length of buffer for dynamic prompt */
 #define READLINE_MAXPROMPT 512
 
-static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
-			    int rw, xiofile_t *xfd, unsigned groups,
-			    int dummy1, int dummy2, int dummy3);
+static int xioopen_readline(int argc, const char *argv[], struct opt *opts, int rw, xiofile_t *xfd, const struct addrdesc *addrdesc);
 
 
-const struct addrdesc addr_readline = {
-   "readline", 3, xioopen_readline, GROUP_FD|GROUP_TERMIOS|GROUP_READLINE, 0, 0, 0 HELP(NULL) };
+const struct addrdesc xioaddr_readline = { "READLINE", 3, xioopen_readline, GROUP_FD|GROUP_TERMIOS|GROUP_READLINE, 0, 0, 0 HELP(NULL) };
 
 const struct optdesc opt_history_file = { "history-file", "history", OPT_HISTORY_FILE, GROUP_READLINE, PH_LATE, TYPE_STRING, OFUNC_OFFSET, XIO_OFFSETOF(para.readline.history_file) };
 const struct optdesc opt_prompt       = { "prompt",       NULL,      OPT_PROMPT,       GROUP_READLINE, PH_LATE, TYPE_STRING, OFUNC_OFFSET, XIO_OFFSETOF(para.readline.prompt) };
 const struct optdesc opt_noprompt     = { "noprompt",     NULL,      OPT_NOPROMPT,     GROUP_READLINE, PH_LATE, TYPE_BOOL,   OFUNC_SPEC,   0 };
 const struct optdesc opt_noecho       = { "noecho",       NULL,      OPT_NOECHO,       GROUP_READLINE, PH_LATE, TYPE_STRING, OFUNC_SPEC,   0 };
 
-static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
-			    int xioflags, xiofile_t *xfd, unsigned groups,
-			    int dummy1, int dummy2, int dummy3) {
+static int xioopen_readline(
+	int argc,
+	const char *argv[],
+	struct opt *opts,
+	int xioflags,
+	xiofile_t *xfd,
+	const struct addrdesc *addrdesc)
+{
+   struct single *sfd = &xfd->stream;
    int rw = (xioflags & XIO_ACCMODE);
    char msgbuf[256], *cp = msgbuf;
    bool noprompt = false;
    char *noecho = NULL;
 
    if (argc != 1) {
-      Error1("%s: 0 parameters required", argv[0]);
+      xio_syntax(argv[0], 0, argc-1, addrdesc->syntax);
       return STAT_NORETRY;
    }
 
@@ -61,8 +64,10 @@ static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
    if ((rw+1)&1) {
       strcpy(cp, "readline on stdin for reading"); cp = strchr(cp, '\0');
 
-      if ((rw+1)&2)
-      strcpy(cp, " and ");  cp = strchr(cp, '\0');
+      if ((rw+1)&2) {
+	 strcpy(cp, " and ");
+	 cp = strchr(cp, '\0');
+      }
    }
    if ((rw+1)&2) {
       strcpy(cp, "stdio for writing"); cp = strchr(cp, '\0');
@@ -70,7 +75,8 @@ static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
    Notice(msgbuf);
 
    xfd->stream.fd = 0;	/* stdin */
-   xfd->stream.howtoend = END_NONE;
+   if (sfd->howtoend == END_UNSPEC)
+      xfd->stream.howtoend = END_NONE;
    xfd->stream.dtype    = XIODATA_READLINE;
 
 #if WITH_TERMIOS
@@ -84,10 +90,10 @@ static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
    }
 #endif /* WITH_TERMIOS */
 
-   if (applyopts_single(&xfd->stream, opts, PH_INIT) < 0)  return -1;
-   applyopts(-1, opts, PH_INIT);
+   if (applyopts_single(sfd, opts, PH_INIT) < 0)  return -1;
+   applyopts(sfd, -1, opts, PH_INIT);
 
-   applyopts2(xfd->stream.fd, opts, PH_INIT, PH_FD);
+   applyopts2(sfd, -1, opts, PH_INIT, PH_FD);
 
    Using_history();
    applyopts_offset(&xfd->stream, opts);
@@ -121,8 +127,8 @@ static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
       Read_history(xfd->stream.para.readline.history_file);
    }
 #if _WITH_TERMIOS
-   xiotermios_clrflag(xfd->stream.fd, 3, ICANON);
-   xiotermios_clrflag(xfd->stream.fd, 3, ECHO);
+   xiotermios_clrflag(xfd->stream.fd, 3, ICANON|ECHO);
+   xiotermios_flush(xfd->stream.fd);
 #endif /* _WITH_TERMIOS */
    return _xio_openlate(&xfd->stream, opts);
 }
@@ -172,14 +178,13 @@ ssize_t xioread_readline(struct single *pipe, void *buff, size_t bufsiz) {
 
 #if _WITH_TERMIOS
       xiotermios_setflag(pipe->fd, 3, ECHO);
+      xiotermios_flush(pipe->fd);
 #endif /* _WITH_TERMIOS */
       if (pipe->para.readline.prompt || pipe->para.readline.dynprompt) {
 	 /* we must carriage return, because readline will first print the
 	    prompt */
 	 ssize_t writt;
-	 do {
-	    writt = Write(pipe->fd, "\r", 1);
-	 } while (writt < 0 && errno == EINTR);
+	 writt = writefull(pipe->fd, "\r", 1, NULL);
 	 if (writt < 0) {
 	    Warn2("write(%d, \"\\r\", 1): %s",
 		   pipe->fd, strerror(errno));
@@ -199,12 +204,16 @@ ssize_t xioread_readline(struct single *pipe, void *buff, size_t bufsiz) {
       if (line == NULL) {
 	 return 0;	/* EOF */
       }
+      if (strlen(line) == 0) {
+	 Write(STDOUT_FILENO, "\n", 1);
+      }
 #if _WITH_TERMIOS
       xiotermios_clrflag(pipe->fd, 3, ECHO);
+      xiotermios_flush(pipe->fd);
 #endif /* _WITH_TERMIOS */
       Add_history(line);
       bytes = strlen(line);
-      strncpy(buff, line, bufsiz);
+      ((char *)buff)[0] = '\0'; strncat(buff, line, bufsiz-1);
       free(line);
       if ((size_t)bytes < bufsiz) {
 	 strcat(buff, "\n");  ++bytes;
@@ -216,25 +225,26 @@ void xioscan_readline(struct single *pipe, const void *buff, size_t bytes) {
    if (pipe->dtype == XIODATA_READLINE && pipe->para.readline.dynprompt) {
       /* we save the last part of the output as possible prompt */
       const void *ptr = buff;
-      const void *pcr = memrchr(buff, '\r', bytes);
-      const void *plf = memrchr(buff, '\n', bytes);
+      const void *pcr;
+      const void *plf;
       size_t len;
+
       if (bytes > pipe->para.readline.dynbytes) {
 	 ptr = (const char *)buff + bytes - pipe->para.readline.dynbytes;
+	 len = pipe->para.readline.dynbytes;
+      } else {
+	 len = bytes;
       }
-      if (pcr) {
+      pcr = memrchr(ptr, '\r', len);
+      plf = memrchr(ptr, '\n', len);
+      if (pcr != NULL || plf != NULL) {
+	 const void *peol = Max(pcr, plf);
 	 /* forget old prompt */
 	 pipe->para.readline.dynend = pipe->para.readline.dynprompt;
+	 len -= (peol+1 - ptr);
 	 /* new prompt starts here */
-	 ptr = (const char *)pcr+1;
+	 ptr = (const char *)peol+1;
       }
-      if (plf && plf >= ptr) {
-	 /* forget old prompt */
-	 pipe->para.readline.dynend = pipe->para.readline.dynprompt;
-	 /* new prompt starts here */
-	 ptr = (const char *)plf+1;
-      }
-      len = (const char *)buff-(const char *)ptr+bytes;
       if (pipe->para.readline.dynend - pipe->para.readline.dynprompt + len >
 	  pipe->para.readline.dynbytes) {
 	 memmove(pipe->para.readline.dynprompt,
@@ -245,7 +255,6 @@ void xioscan_readline(struct single *pipe, const void *buff, size_t bytes) {
 	    pipe->para.readline.dynprompt + pipe->para.readline.dynbytes - len;
       }
       memcpy(pipe->para.readline.dynend, ptr, len);
-      /*pipe->para.readline.dynend = pipe->para.readline.dynprompt + len;*/
       pipe->para.readline.dynend = pipe->para.readline.dynend + len;
    }
    return;
