@@ -10,25 +10,19 @@ struct socks4 {
    uint8_t  action;
    uint16_t port;
    uint32_t dest;
-   char userid[1];	/* just to have access via this struct */
+   char userid[0];	/* just to have access via this struct */
 } ;
-#define SIZEOF_STRUCT_SOCKS4 8
+#define SIZEOF_STRUCT_SOCKS4 ((size_t)&((struct socks4 *)0)->userid)
 
 extern const struct optdesc opt_socksport;
 extern const struct optdesc opt_socksuser;
 
-extern const struct addrdesc addr_socks4_connect;
-extern const struct addrdesc addr_socks4a_connect;
+extern const struct addrdesc xioaddr_socks4_connect;
+extern const struct addrdesc xioaddr_socks4a_connect;
 
-extern int _xioopen_socks4_prepare(const char *targetport, struct opt *opts, char **socksport, struct socks4 *sockhead, size_t *headlen);
-extern int
-   _xioopen_socks4_connect0(struct single *xfd,
-			    const char *hostname,	/* socks target host */
-			    int socks4a,
-			    struct socks4 *sockhead,
-			    ssize_t *headlen,		/* get available space,
-							   return used length*/
-			    int level);
+extern int _xioopen_opt_socksport(struct opt *opts, char **socksport);
+extern int _xioopen_socks4_init(const char *targetport, struct opt *opts, char **socksport, struct socks4 *sockhead, size_t *headlen);
+extern int _xioopen_socks4_prepare(struct single *xfd, const char *hostname, int socks4a, struct socks4 *sockhead, ssize_t *headlen, int level);
 extern int _xioopen_socks4_connect(struct single *xfd,
 				   struct socks4 *sockhead,
 				   size_t headlen,
